(function($) {

    var pluginName = "nestedFormset";

    var NestedFormset = Class.extend({
        opts: {
            emptyClass: 'empty-form grp-empty-form',
            predeleteClass: 'grp-predelete'
        },
        init: function(inline) {
            this.$inline = $(inline);
            this.prefix = this.$inline.djangoFormsetPrefix();
            this._$totalForms = this.$inline.find('#id_' + this.prefix + '-TOTAL_FORMS');
            this._$totalForms.attr('autocomplete', 'off');
            this._$template = $('#' + this.prefix + '-empty');

            var inlineModelClassName = this.$inline.data('inlineModel');

            this.opts = $.extend({}, this.opts, {
                addButtonSelector: '.add-handler.' + inlineModelClassName,
                removeButtonSelector: '.remove-handler.' + inlineModelClassName,
                deleteButtonSelector: '.delete-handler.' + inlineModelClassName,
                formClass: 'dynamic-form-' + inlineModelClassName
            });

            DJNesting.initRelatedFields(this.prefix, this.$inline.data());
            DJNesting.initAutocompleteFields(this.prefix, this.$inline.data());

            this._bindEvents();

            this._initializeForms();

            this.$inline.find('.nested-sortable-container:not([id*="-empty"])').trigger('djnesting:init');

            // initialize nested formsets
            this.$inline.find('.group[id$="-group"][id^="' + this.prefix + '"][data-field-names]:not([id*="-empty"])').each(function() {
                $(this)[pluginName]();
            });

            if (this.$inline.is('.djnesting-stacked-root')) {
                DJNesting.createSortable(this.$inline);
            }

            $(document).trigger('djnesting:initialized', [this.$inline, this]);
        },
        _initializeForms: function() {
            var totalForms = this.mgmtVal('TOTAL_FORMS');
            for (var i = 0; i < totalForms; i++) {
                this._initializeForm('#' + this.prefix + i);
            }
        },
        _initializeForm: function(form) {
            var $form = $(form);
            var formPrefix = $form.djangoFormPrefix();
            $form.addClass(this.opts.formClass);
            if ($form.hasClass('has_original')) {
                $('#id_' + formPrefix + 'DELETE:checked').toggleClass(this.opts.predeleteClass);
            }
        },
        _bindEvents: function($el) {
            var self = this;
            if (typeof $el == 'undefined') {
                $el = this.$inline;
            }
            $el.find(this.opts.addButtonSelector).click(function(e) {
                e.preventDefault();
                e.stopPropagation();
                self.add();
            });
            $el.find(this.opts.removeButtonSelector).click(function(e) {
                e.preventDefault();
                e.stopPropagation();
                var $form = $(this).closest('.' + self.opts.formClass);
                self.remove($form);
                $(document).trigger('djnesting:mutate', [this.$inline]);
            });
            $el.find(this.opts.deleteButtonSelector).click(function(e) {
                e.preventDefault();
                e.stopPropagation();
                var $form = $(this).closest('.' + self.opts.formClass);
                var $deleteInput = $('#id_' + $form.djangoFormPrefix() + 'DELETE');
                if (!$deleteInput.is(':checked')) {
                    self['delete']($form);
                } else {
                    self.undelete($form);
                }
            });
        },
        _afterRemove: function(form) {
            var self = this;
            var $form = $(form);
            var totalForms = this.mgmtVal('TOTAL_FORMS');
            var initialForms = this.mgmtVal('INITIAL_FORMS');
            // var initialForms = this.$inline.djangoFormsetForms().filter('[data-is-initial="true"]').length;
            var maxForms = this.mgmtVal('MAX_NUM_FORMS') || Infinity;
            var formId = $form.attr('id');

            this.mgmtVal('TOTAL_FORMS', totalForms - 1);

            if (maxForms - totalForms >= 0) {
                this.$inline.find(this.opts.addButtonSelector).parents('.grp-add-item').show();
            }

            var form_re = new RegExp(DJNesting.regexQuote(this.prefix) + '\\-\\d+');

            // Re-index the non-initial form attributes
            for (var i = totalForms - 1; i >= initialForms; i--) {
                var $f = $('#' + this.prefix + i);
                if (!$f[0] || !$f[0].parentNode || !$f[0].parentNode.parentNode) {
                    break;
                }
                if ($f.attr('id') == formId) {
                    break;
                }
                newIndex = i - 1;
                oldFormPrefixRegex = new RegExp("^(id_)?"
                    + DJNesting.regexQuote(this.prefix + "-" + i));
                $f.attr('id', this.prefix + newIndex);
                DJNesting.updateFormAttributes($f, oldFormPrefixRegex, "$1" + this.prefix + "-" + newIndex);
                $(document).trigger('djnesting:attrchange', [this.$inline, $f]);
                if (formId === this.prefix + i) {
                    break;
                }
            }

            DJNesting.updatePositions(this.prefix);
        },
        remove: function(form) {
            var $form = $(form);

            if ($form[0].parentNode && $form[0].parentNode.parentNode) {
                $form.parent().remove();
            }

            this._afterRemove($form);
        },
        "delete": function(form) {
            var $form = $(form),
                formPrefix = $form.djangoFormPrefix(),
                $deleteInput = $('#id_' + formPrefix + 'DELETE');

            if ($form.parent().closest('.' + this.opts.predeleteClass).length) {
                return;
            }

            if ($form.hasClass('has_original')) {
                $deleteInput.attr('checked', 'checked');
                $form.addClass(this.opts.predeleteClass);
            }

            DJNesting.updatePositions(this.prefix);
            $(document).trigger('djnesting:mutate', [this.$inline]);
        },
        undelete: function(form) {
            var $form = $(form),
                formPrefix = $form.djangoFormPrefix(),
                $deleteInput = $('#id_' + formPrefix + 'DELETE');

            if ($form.parent().closest('.' + this.opts.predeleteClass).length) {
                return;
            }
            if ($form.hasClass('has_original')) {
                $deleteInput.removeAttr('checked');
                $form.removeClass(this.opts.predeleteClass);
            }

            DJNesting.updatePositions(this.prefix);
            $(document).trigger('djnesting:mutate', [this.$inline]);
        },
        add: function() {
            var self = this;
            var $form = this._$template.clone();
            var index = this.mgmtVal('TOTAL_FORMS');
            var maxForms = this.mgmtVal('MAX_NUM_FORMS') || Infinity;
            var isNested = this.$inline.hasClass('djnesting-stacked');

            $form.removeClass(this.opts.emptyClass);
            $form.attr('id', $form.attr('id').replace("-empty", index));

            if (isNested) {
                $form.wrap('<div class="nested-sortable-item"></div>').parent().insertBefore(this._$template.parent());
                $form.parent().append(DJNesting.createContainerElement());
            } else {
                $form.insertBefore(this._$template);
                $form.parent.addClass('nested-sortable-item');
            }

            this.mgmtVal('TOTAL_FORMS', index + 1);
            if (maxForms - index <= 0) {
                this.$inline.find(this.opts.addButtonSelector).parents('.grp-add-item').hide();
            }

            DJNesting.updateFormAttributes($form, /__prefix__/, index);

            DJNesting.updateNestedFormIndex($form, this.prefix);
            DJNesting.updatePositions(this.prefix);

            grappelli.reinitDateTimeFields($form);
            grappelli.updateSelectFilter($form);
            DJNesting.initRelatedFields(this.prefix);
            DJNesting.initAutocompleteFields(this.prefix);
            $form.find(".collapse").andSelf().grp_collapsible({
                toggle_handler_slctr: ".collapse-handler:first",
                closed_css: "closed grp-closed",
                open_css: "open grp-open",
                on_toggle: function(elem, options) {
                    $(document).trigger('djnesting:toggle', [self.$inline]);
                }
            });
            if (typeof $.fn.curated_content_type == 'function') {
                $form.find('.curated-content-type-select').each(function(index, element) {
                    $(element).curated_content_type();
                });
            }

            this._initializeForm($form);
            this._bindEvents($form);

            // find any nested formsets
            $form.find('.group[id$="-group"][id^="' + this.prefix + '"][data-field-names]:not([id*="-empty"])').each(function() {
                $(this)[pluginName]();
            });

            // Fire an event on the document so other javascript applications
            // can be alerted to the newly inserted inline
            $(document).trigger('djnesting:added', [this.$inline, $form]);

            return $form;
        },
        _moveForm: function($form, prevIndex, index, $prevInline) {
            // debugger;
            var formId = $form.attr('id');
            var $lastFormInPrevInline = $prevInline.find('> .nested-sortable-container > .nested-sortable-item').children('.nested-inline-form[data-is-initial="true"]:not([id="' + formId + '"])').last();
            // $form.parent().nextAll('.nested-sortable-item').children('.nested-inline-form[data-is-initial="true"]').last();
            var needsRemoval = ($form.closest('.djnesting-stacked').attr('id') == $prevInline.attr('id'));
            if (needsRemoval) {
                $item = $form.parent().remove();
            }
            if ($lastFormInPrevInline.length) {
                var lastFormIndex = $lastFormInPrevInline.djangoFormIndex();
                if (lastFormIndex > prevIndex) {
                    var prevFormsetPrefix = $form.djangoFormsetPrefix();
                    prevFormPrefixRegex = new RegExp("^(id_)?"
                        + DJNesting.regexQuote(prevFormsetPrefix + "-" + lastFormIndex));
                    $lastFormInPrevInline.attr('id', prevFormsetPrefix + prevIndex);
                    DJNesting.updateFormAttributes($lastFormInPrevInline, prevFormPrefixRegex, "$1" + prevFormsetPrefix + "-" + prevIndex);
                    $(document).trigger('djnesting:attrchange', [this.$inline, $lastFormInPrevInline]);
                }
            }
            $prevInline.nestedFormset()._afterRemove($form);
            if (needsRemoval) {
                $(document).trigger('djnesting:removed', [$prevInline, $form]);
                $before = this.$inline.find('> .nested-sortable-container').find('.nested-sortable-item').eq(index);
                $before.after($item);
            }
        },
        /**
         * Splice a form into the current formset at new position `index`.
         */
        spliceInto: function($form, index) {
            var initialFormCount = this.mgmtVal('INITIAL_FORMS'),
                totalFormCount = this.mgmtVal('TOTAL_FORMS'),
                oldFormsetPrefix = $form.djangoFormsetPrefix(),
                newFormsetPrefix = this.prefix,
                isInitial = $form.data('isInitial'),
                newIndex, $item, $before,
                wasAdded = false;
            var currentPosition = $form.parent().prevAll('.nested-sortable-item:not(.nested-do-not-drag)').length;

            // Make sure the form being spliced is from a different inline
            if ($form.djangoFormsetPrefix() == this.prefix) {
                if (currentPosition === index) {
                    return;
                }
                $item = $($form[0].parentNode.parentNode.removeChild($form[0].parentNode));
                $before = this.$inline.find('> .nested-sortable-container').find('.nested-sortable-item').eq(index);
                $before.after($item);
            } else {
                // debugger;
                var $oldInline = $('#' + oldFormsetPrefix + '-group');
                var $currentFormInline = $form.closest('.djnesting-stacked');

                // if ($currentFormInline.djangoFormsetPrefix() != newFormsetPrefix) {
                this._moveForm($form, currentPosition, index, $oldInline);
                wasAdded = true;
                // }

                var oldNestedFormset = $oldInline.nestedFormset();
                if (!wasAdded) {
                    oldNestedFormset.mgmtVal('TOTAL_FORMS', oldNestedFormset.mgmtVal('TOTAL_FORMS') - 1);
                }

                if (isInitial) {
                    var $parentInline = this.$inline.parent().closest('.djnesting-stacked');
                    if ($parentInline.length) {
                        var $parentForm = this.$inline.closest('.nested-inline-form');
                        var parentPkField = $parentInline.data('fieldNames').pk;
                        var $parentPk = $parentForm.djangoFormField(parentPkField);
                        if (!$parentPk.val()) {
                            isInitial = false;
                        }
                    }
                }
                if (isInitial) {
                    oldNestedFormset.mgmtVal('INITIAL_FORMS', oldNestedFormset.mgmtVal('INITIAL_FORMS') - 1);

                    // Re-index the non-initial form attributes
                    for (i = totalFormCount - 1; i >= initialFormCount; i--) {
                        var $f = $('#' + newFormsetPrefix + i);
                        newIndex = i + 1;
                        oldFormPrefixRegex = new RegExp("^(id_)?"
                            + DJNesting.regexQuote(newFormsetPrefix + "-" + i));
                        $f.attr('id', newFormsetPrefix + newIndex);
                        DJNesting.updateFormAttributes($f, oldFormPrefixRegex, "$1" + newFormsetPrefix + "-" + newIndex);
                        $(document).trigger('djnesting:attrchange', [this.$inline, $f]);
                    }
                }

                // Replace the ids for the splice form
                oldFormPrefixRegex = new RegExp("^(id_)?"
                    + DJNesting.regexQuote($form.attr('id').replace(/_set(\d+)$/, '_set-$1')));
                newIndex = (isInitial) ? initialFormCount : totalFormCount;
                $form.attr('id', newFormsetPrefix + newIndex);
                DJNesting.updateFormAttributes($form, oldFormPrefixRegex, "$1" + newFormsetPrefix + "-" + newIndex);
                $(document).trigger('djnesting:attrchange', [this.$inline, $form]);

                if (isInitial) {
                    this.mgmtVal('INITIAL_FORMS', initialFormCount + 1);
                }
                this.mgmtVal('TOTAL_FORMS', totalFormCount + 1);

                DJNesting.updatePositions(oldFormsetPrefix);
            }

            DJNesting.updatePositions(newFormsetPrefix);
            if (wasAdded) {
                $(document).trigger('djnesting:added', [this.$inline, $form]);
            }
            $(document).trigger('djnesting:mutate', [this.$inline]);
        },
        mgmtVal: function(name, newValue) {
            var $field = this.$inline.find('#id_' + this.prefix + '-' + name);
            if (typeof newValue == 'undefined') {
                return parseInt($field.val(), 10);
            } else {
                return parseInt($field.val(newValue).val(), 10);
            }
        }
    });

    $.fn[pluginName] = function() {
        var options, fn, args;
        var $el = this.eq(0);

        if (arguments.length === 0 || (arguments.length === 1 && $.type(arguments[0]) != 'string')) {
            options = arguments[0];
            var nestedFormset = $el.data(pluginName);
            if (!nestedFormset) {
                nestedFormset = new NestedFormset($el, options);
                $el.data(pluginName, nestedFormset);
            }
            return nestedFormset;
        }

        fn = arguments[0];
        args = $.makeArray(arguments).slice(1);

        if (fn in NestedFormset.prototype) {
            return $el.data(pluginName)[fn](args);
        } else {
            throw new Error("Unknown function call " + fn + " for $.fn." + pluginName);
        }
    };

})((typeof grp == 'object' && grp.jQuery) ? grp.jQuery : django.jQuery);
